#include "streamer-private.h"

GSList *_StreamPlugin_list;

void _StreamPlugin_init() {
  DEBUG("in _StreamPlugin_init()\n");
  _StreamPlugin_list = NULL;
}

StreamPlugin *StreamPlugin_load(gchar *name) {
  GSList *loaded_plugin;
  gchar *libname;
  gchar *module_name;
  GModule *module = NULL;
  void (*initfunc)(StreamPlugin *);
  StreamPlugin *plugin;

  /* insanity check */
  if (!name) {
    g_warning("StreamPlugin_load: no plugin name given\n");
    return;
  }

  /* if it's not an absolute pathname or specifically a .so */
  if (!strchr(name,'/') && !strstr(name,".so")) {
    /* attempt to load it from normal paths */
    libname = g_strdup_printf("lib%s.so",name);
#ifdef ELEMENTS_USE_SRCDIR
    /* attempt to find the plugin in the local build dir first */
    module_name = g_module_build_path(ELEMENTS_SRCDIR,libname);
    loaded_plugin = g_slist_find_custom(_StreamPlugin_list,module_name,
                                        _StreamPlugin_matchbyname);
    if (loaded_plugin) {
      DEBUG("StreamPlugin_load: requested plugin already loaded\n");
      g_free(module_name);
      g_free(libname);
      return((StreamPlugin *)loaded_plugin->data);
    }
    DEBUG("StreamPlugin_load: attempting to load %s\n",module_name);
    module = g_module_open(module_name,0);
    if (!module) DEBUG("StreamPlugin_load: %s\n",g_module_error());
#endif /* ELEMENTS_USE_SRCDIR */
    /* attempt to find the plugin in the normal installed location */
    if (!module) {
      module_name = g_module_build_path(ELEMENTS_DIR,libname);
      loaded_plugin = g_slist_find_custom(_StreamPlugin_list,module_name,
                                          _StreamPlugin_matchbyname);
      if (loaded_plugin) {
        DEBUG("StreamPlugin_load: requested plugin already loaded\n");
        g_free(module_name);
        g_free(libname);
        return((StreamPlugin *)loaded_plugin->data);
      }
      DEBUG("StreamPlugin_load: attempting to load %s\n",module_name);
      module = g_module_open(module_name,0);
      if (!module) DEBUG("StreamPlugin_load: %s\n",g_module_error());
    }
  } else {
    /* it's supposedly an absolute path, let's try it... */
    module_name = g_strdup(name);
    module = g_module_open(module_name,0);
    if (!module) DEBUG("StreamPlugin_load: %s\n",g_module_error());
    DEBUG("StreamPlugin_load: attempting to load %s\n",module_name);
  }

  /* if it's not there, we've got a problem */
  if (!module) {
    /* FIXME: this is waaaaaaay too harsh */
    g_error("failed to load %s\n",name);
    exit(1);
  }

  /* look up the initialization function */
  DEBUG("StreamPlugin_load: attempting to look up symbol init_StreamPlugin\n");
  if (!g_module_symbol(module,"init_StreamPlugin",(gpointer)&initfunc)) {
    /* FIXME: this is also way to harsh */
    g_error("no initialization function in plugin %s\n",name);\
    exit(1);
  }

  /* now create a StreamPlugin to keep track of the poor thing */
  plugin = (StreamPlugin *)g_malloc(sizeof(StreamPlugin));
  plugin->filename = module_name;
  plugin->module = module;
  plugin->initfunc = initfunc;

  /* call the module_init function to let it actually init stuff */
  (plugin->initfunc)(plugin);

  /* add it to the global list */
  DEBUG("StreamPlugin_load: adding plugin %s to list\n",name);
  _StreamPlugin_list = g_slist_append(_StreamPlugin_list,plugin);

  /* looks like we're done, we can return the Stream_Plugin now */
  return(plugin);
}

gchar *StreamPlugin_getname(StreamPlugin *plugin) {
  return(plugin->name);
}

gboolean _StreamPlugin_matchbyname(gconstpointer a, gconstpointer b) {
  DEBUG("StreamPlugin_matchbyname: comparing %s to %s\n",
        ((StreamPlugin *)a)->filename,(gchar *)b);
  return (strcmp(((StreamPlugin *)a)->filename,(gchar *)b));
}

StreamElementType *StreamPlugin_enumtypes(StreamPlugin *plugin,int idx) {
  GSList *types;
  int i;

  /* insanity check */
  if (!plugin) {
    g_warning("StreamPlugin_enumtypes: NULL plugin!\n");
    return(NULL);
  }

  if (idx < 0) {
    g_warning("StreamPlugin_enumtypes: invalid index\n");
    return(NULL);
  }

  types = plugin->elementtypes;
  if (!types) {
    g_warning("StreamPlugin_enumtypes: no element types registered\n");
    return(NULL);
  }

/*  for (i=0;i<idx;i++) {
    types = g_slist_next(types);
    if (!types) {
      g_warning("StreamPlugin_enumtypes: invalid index\n");
      return(NULL);
    }
  }*/

  return((StreamElementType *)g_slist_nth_data(types,idx));
}
